/*
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.googlecode.gwtmeasure.server.servlet;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;
import com.googlecode.gwtmeasure.server.MeasureContext;
import com.googlecode.gwtmeasure.server.MetricsProcessor;
import com.googlecode.gwtmeasure.server.internal.NetworkEventProducer;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @author <a href="dmitry.buzdin@ctco.lv">Dmitry Buzdin</a>
 */
public class MeasuringRemoteServiceServlet extends RemoteServiceServlet {

    private MetricsProcessor metricsProcessor;
    private NetworkEventProducer networkEventProducer;

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        MeasureContext context = MeasureContext.instance();
        metricsProcessor = context.getBean(MetricsProcessor.class);
        networkEventProducer = context.getBean(NetworkEventProducer.class);
    }

    @Override
    protected void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        if (!metricsProcessor.isProcessed(request)) {            
            networkEventProducer.requestReceived(request);
            metricsProcessor.extractAndProcess(request);
            metricsProcessor.markAsProcessed(request);
            super.service(request, response);            
            networkEventProducer.reponseSent(request);
        } else {
            super.service(request, response);
        }
    }

    @Override
    public void destroy() {
        super.destroy();
        MeasureContext.instance().reset();
    }

}
